@@ -27,6 +27,9 @@ public:
 	void testCompare();
 	void testTransform();
 
+	void testEscape();
+	void testUnescape();
+
 	void setUp();
 	void tearDown();
 
